@@ -1,6 +1,5 @@
 from urllib import parse
-from datetime import datetime
-from typing import Dict, List, Optional
+from typing import Dict, Optional
 
 
 from tea_client.http import HttpClient
@@ -10,7 +9,7 @@ from paperswithcode.config import config
 from paperswithcode.models import (
     Paper,
     Papers,
-    Repository,
+    Repositories,
     Conference,
     Conferences,
     Proceeding,
@@ -28,9 +27,11 @@ from paperswithcode.models import (
     Method,
     Methods,
     Metric,
+    Metrics,
     MetricCreateRequest,
     MetricUpdateRequest,
     Result,
+    Results,
     ResultCreateRequest,
     ResultUpdateRequest,
     EvaluationTable,
@@ -69,7 +70,7 @@ class PapersWithCodeClient:
             return 1
         else:
             q = parse.parse_qs(p.query)
-            return q["page"][0]
+            return q.get("page", [1])[0]
 
     @classmethod
     def __page(cls, result, page_model):
@@ -86,22 +87,24 @@ class PapersWithCodeClient:
             results=result["results"],
         )
 
-    @staticmethod
-    def __create_result(data: dict) -> Result:
-        return Result(**data)
-
-    @staticmethod
-    def __create_result_sync_data(data: dict) -> dict:
-        return data
-
     @handler
     def paper_list(
-        self, q: Optional[str] = None, page: int = 1, items_per_page: int = 50
+        self,
+        q: Optional[str] = None,
+        arxiv_id: Optional[str] = None,
+        title: Optional[str] = None,
+        abstract: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
     ) -> Papers:
         """Return a paginated list of papers.
 
         Args:
-            q (str): Filter papers by querying the paper title and abstract.
+            q (str, optional): Filter papers by querying the paper title and
+                abstract.
+            arxiv_id (str, optional): Filter papers by arxiv id.
+            title (str, optional): Filter papers by part of the title.
+            abstract (str, optional): Filter papers by part of the abstract.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -114,6 +117,13 @@ class PapersWithCodeClient:
         if q is not None:
             params["q"] = q
             timeout = 60
+        if arxiv_id is not None:
+            params["arxiv_id"] = arxiv_id
+        if title is not None:
+            params["title"] = title
+        if abstract is not None:
+            params["abstract"] = abstract
+            timeout = 60
         return self.__page(
             self.http.get("/papers/", params=params, timeout=timeout), Papers
         )
@@ -131,68 +141,72 @@ class PapersWithCodeClient:
         return Paper(**self.http.get(f"/papers/{paper_id}/"))
 
     @handler
-    def paper_repository_list(self, paper_id: str) -> List[Repository]:
+    def paper_repository_list(self, paper_id: str) -> Repositories:
         """Return a list of paper implementations.
 
         Args:
             paper_id (str): ID of the paper.
 
         Returns:
-            List[Repository]: List of repository objects.
+            Repositories: Repositories object.
         """
-        return [
-            Repository(**r)
-            for r in self.http.get(f"/papers/{paper_id}/repositories/")
-        ]
+        return self.__page(
+            self.http.get(f"/papers/{paper_id}/repositories/"), Repositories
+        )
 
     @handler
-    def paper_task_list(self, paper_id: str) -> List[Task]:
+    def paper_task_list(self, paper_id: str) -> Tasks:
         """Return a list of tasks mentioned in the paper.
 
         Args:
             paper_id (str): ID of the paper.
 
         Returns:
-            List[Task]: List of task objects.
+            Tasks: Tasks object.
         """
-        return [Task(**t) for t in self.http.get(f"/papers/{paper_id}/tasks/")]
+        return self.__page(self.http.get(f"/papers/{paper_id}/tasks/"), Tasks)
 
     @handler
-    def paper_method_list(self, paper_id: str) -> List[Method]:
+    def paper_method_list(self, paper_id: str) -> Methods:
         """Return a list of methods mentioned in the paper.
 
         Args:
             paper_id (str): ID of the paper.
 
         Returns:
-            List[Method]: List of method objects.
+            Methods: Methods object.
         """
-        return [
-            Method(**t) for t in self.http.get(f"/papers/{paper_id}/methods/")
-        ]
+        return self.__page(
+            self.http.get(f"/papers/{paper_id}/methods/"), Methods
+        )
 
     @handler
-    def paper_result_list(self, paper_id: str) -> List[Result]:
+    def paper_result_list(self, paper_id: str) -> Results:
         """Return a list of evaluation results for the paper.
 
         Args:
             paper_id (str): ID of the paper.
 
         Returns:
-            List[Result]: List of result objects.
+            Results: Results object.
         """
-        return [
-            self.__create_result(result)
-            for result in self.http.get(f"/papers/{paper_id}/results/")
-        ]
+        return self.__page(
+            self.http.get(f"/papers/{paper_id}/results/"), Results
+        )
 
     @handler
     def conference_list(
-        self, page: int = 1, items_per_page: int = 50
+        self,
+        q: Optional[str] = None,
+        name: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
     ) -> Conferences:
         """Return a paginated list of conferences.
 
         Args:
+            q (str, optional): Search all searchable fields.
+            name (str, optional): Filter conferences by part of the name.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -200,10 +214,13 @@ class PapersWithCodeClient:
         Returns:
             Conferences: Conferences object.
         """
+        params = self.__params(page, items_per_page)
+        if q is not None:
+            params["q"] = q
+        if name is not None:
+            params["name"] = name
         return self.__page(
-            self.http.get(
-                "/conferences/", params=self.__params(page, items_per_page)
-            ),
+            self.http.get("/conferences/", params=params),
             Conferences,
         )
 
@@ -263,33 +280,47 @@ class PapersWithCodeClient:
 
     @handler
     def proceeding_paper_list(
-        self, conference_id: str, proceeding_id: str
-    ) -> List[Paper]:
+        self,
+        conference_id: str,
+        proceeding_id: str,
+        page: int = 1,
+        items_per_page: int = 50,
+    ) -> Papers:
         """Return a list of papers published in a confernce proceeding.
 
         Args:
             conference_id (str): ID of the conference.
             proceeding_id (str): ID of the proceding.
+            page (int): Desired page.
+            items_per_page (int): Desired number of items per page.
+                Default: 50.
 
         Returns:
-            List[Paper]: List of paper objects.
+            Papers: Papers object.
         """
-        return [
-            Paper(**p)
-            for p in self.http.get(
+        params = self.__params(page, items_per_page)
+        return self.__page(
+            self.http.get(
                 f"/conferences/{conference_id}/proceedings/{proceeding_id}"
-                f"/papers/"
-            )
-        ]
+                f"/papers/",
+                params=params,
+            ),
+            Papers,
+        )
 
     @handler
     def area_list(
-        self, q: Optional[str] = None, page: int = 1, items_per_page: int = 50
+        self,
+        q: Optional[str] = None,
+        name: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
     ) -> Areas:
         """Return a paginated list of areas.
 
         Args:
-            q (str): Filter areas by querying the area name.
+            q (str, optional): Filter areas by querying the area name.
+            name (str, optional): Filter areas by part of the name.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -302,6 +333,9 @@ class PapersWithCodeClient:
         if q is not None:
             params["q"] = q
             timeout = 60
+        if name is not None:
+            params["name"] = name
+
         return self.__page(
             self.http.get("/areas/", params=params, timeout=timeout), Areas
         )
@@ -343,12 +377,17 @@ class PapersWithCodeClient:
 
     @handler
     def task_list(
-        self, q: Optional[str] = None, page: int = 1, items_per_page: int = 50
+        self,
+        q: Optional[str] = None,
+        name: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
     ) -> Tasks:
         """Return a paginated list of tasks.
 
         Args:
-            q (str): Filter tasks by querying the task name.
+            q (str, optional): Filter tasks by querying the task name.
+            name (str, optional): Filter tasks by part of th name.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -361,6 +400,8 @@ class PapersWithCodeClient:
         if q is not None:
             params["q"] = q
             timeout = 60
+        if name is not None:
+            params["name"] = name
         return self.__page(
             self.http.get("/tasks/", params=params, timeout=timeout), Tasks
         )
@@ -435,28 +476,34 @@ class PapersWithCodeClient:
         )
 
     @handler
-    def task_evaluation_list(self, task_id: str) -> List[EvaluationTable]:
+    def task_evaluation_list(self, task_id: str) -> EvaluationTables:
         """Return a list of evaluation tables for a selected task.
 
         Args:
             task_id (str): ID of the task.
 
         Returns:
-            List[EvaluationTable]: List of short evaluation table objects.
+            EvaluationTables: EvaluationTables object.
         """
-        return [
-            EvaluationTable(**sp)
-            for sp in self.http.get(f"/tasks/{task_id}/evaluations/")
-        ]
+        return self.__page(
+            self.http.get(f"/tasks/{task_id}/evaluations/"), EvaluationTables
+        )
 
     @handler
     def dataset_list(
-        self, q: Optional[str] = None, page: int = 1, items_per_page: int = 50
+        self,
+        q: Optional[str] = None,
+        name: Optional[str] = None,
+        full_name: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
     ) -> Datasets:
         """Return a paginated list of datasets.
 
         Args:
-            q (str): Filter datasets by querying the dataset name.
+            q (str, optional): Filter datasets by querying the dataset name.
+            name (str, optional): Filter datasets by their name.
+            full_name (str, optional): Filter datasets by their full name.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -469,6 +516,10 @@ class PapersWithCodeClient:
         if q is not None:
             params["q"] = q
             timeout = 60
+        if name is not None:
+            params["name"] = name
+        if full_name is not None:
+            params["full_name"] = full_name
         return self.__page(
             self.http.get("/datasets/", params=params, timeout=timeout),
             Datasets,
@@ -525,27 +576,35 @@ class PapersWithCodeClient:
         self.http.delete(f"/datasets/{dataset_id}/")
 
     @handler
-    def dataset_evaluation_list(
-        self, dataset_id: str
-    ) -> List[EvaluationTable]:
+    def dataset_evaluation_list(self, dataset_id: str) -> EvaluationTables:
         """Return a list of evaluation tables for a selected dataset.
 
         Args:
             dataset_id (str): ID of the dasaset.
 
         Returns:
-            List[EvaluationTable]: List of short evaluation table objects.
+           EvaluationTables: EvaluationTables object.
         """
-        return [
-            EvaluationTable(**sp)
-            for sp in self.http.get(f"/datasets/{dataset_id}/evaluations/")
-        ]
+        return self.__page(
+            self.http.get(f"/datasets/{dataset_id}/evaluations/"),
+            EvaluationTables,
+        )
 
     @handler
-    def method_list(self, page: int = 1, items_per_page: int = 50) -> Methods:
+    def method_list(
+        self,
+        q: Optional[str] = None,
+        name: Optional[str] = None,
+        full_name: Optional[str] = None,
+        page: int = 1,
+        items_per_page: int = 50,
+    ) -> Methods:
         """Return a paginated list of methods.
 
         Args:
+            q (str, optional): Search all searchable fields.
+            name (str, optional): Filter methods by part of the name.
+            full_name (str, optional): Filter methods by part of the full name.
             page (int): Desired page.
             items_per_page (int): Desired number of items per page.
                 Default: 50.
@@ -553,10 +612,17 @@ class PapersWithCodeClient:
         Returns:
             Methods: Methods object.
         """
+        params = self.__params(page, items_per_page)
+        timeout = None
+        if q is not None:
+            params["q"] = q
+            timeout = 60
+        if name is not None:
+            params["name"] = name
+        if full_name is not None:
+            params["full_name"] = full_name
         return self.__page(
-            self.http.get(
-                "/methods/", params=self.__params(page, items_per_page)
-            ),
+            self.http.get("/methods/", params=params, timeout=timeout),
             Methods,
         )
 
@@ -654,19 +720,18 @@ class PapersWithCodeClient:
         self.http.delete(f"/evaluations/{evaluation_id}/")
 
     @handler
-    def evaluation_metric_list(self, evaluation_id: str) -> List[Metric]:
+    def evaluation_metric_list(self, evaluation_id: str) -> Metrics:
         """List all metrics used in the evaluation table.
 
         Args:
             evaluation_id (str): ID of the evaluation table.
 
         Returns:
-            List[Metric]: All metrics used in the evaluation table.
+            Metrics: Metrics object.
         """
-        return [
-            Metric(**m)
-            for m in self.http.get(f"/evaluations/{evaluation_id}/metrics/")
-        ]
+        return self.__page(
+            self.http.get(f"/evaluations/{evaluation_id}/metrics/"), Metrics
+        )
 
     @handler
     def evaluation_metric_get(
@@ -738,21 +803,18 @@ class PapersWithCodeClient:
         self.http.delete(f"/evaluations/{evaluation_id}/metrics/{metric_id}/")
 
     @handler
-    def evaluation_result_list(self, evaluation_id: str) -> List[Result]:
+    def evaluation_result_list(self, evaluation_id: str) -> Results:
         """List all results from the evaluation table.
 
         Args:
             evaluation_id (str): ID of the evaluation table.
 
         Returns:
-            List[Result]: All results from the evaluation table.
+            Results: Results object.
         """
-        return [
-            self.__create_result(result)
-            for result in self.http.get(
-                f"/evaluations/{evaluation_id}/results/"
-            )
-        ]
+        return self.__page(
+            self.http.get(f"/evaluations/{evaluation_id}/results/"), Results
+        )
 
     @handler
     def evaluation_result_get(
@@ -767,8 +829,10 @@ class PapersWithCodeClient:
         Returns:
             Result: Requested result.
         """
-        return self.__create_result(
-            self.http.get(f"/evaluations/{evaluation_id}/results/{result_id}/")
+        return Result(
+            **self.http.get(
+                f"/evaluations/{evaluation_id}/results/{result_id}/"
+            )
         )
 
     @handler
@@ -784,8 +848,8 @@ class PapersWithCodeClient:
         Returns:
             Result: Created result.
         """
-        return self.__create_result(
-            self.http.post(
+        return Result(
+            **self.http.post(
                 f"/evaluations/{evaluation_id}/results/", data=result
             )
         )
@@ -804,8 +868,8 @@ class PapersWithCodeClient:
         Returns:
             Result: Updated result.
         """
-        return self.__create_result(
-            self.http.patch(
+        return Result(
+            **self.http.patch(
                 f"/evaluations/{evaluation_id}/results/{result_id}/",
                 data=result,
             )
@@ -826,7 +890,5 @@ class PapersWithCodeClient:
         self, evaluation: EvaluationTableSyncRequest
     ) -> EvaluationTableSyncResponse:
         d = self.http.post("/rpc/evaluation-synchronize/", data=evaluation)
-        d["results"] = [
-            self.__create_result_sync_data(result) for result in d["results"]
-        ]
+        d["results"] = [result for result in d["results"]]
         return EvaluationTableSyncResponse(**d)
